==> v1.13.zip: combo/combo/apps/wcs/forms.py <==
# combo - content management system
# Copyright (C) 2014-2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.utils.datastructures import MultiValueDict
from django.utils.safestring import mark_safe

from .models import WcsFormCell, WcsCategoryCell, WcsFormsOfCategoryCell
from .utils import get_wcs_options

class WcsFormCellForm(forms.ModelForm):
    class Meta:
        model = WcsFormCell
        fields = ('formdef_reference',)

    def __init__(self, *args, **kwargs):
        super(WcsFormCellForm, self).__init__(*args, **kwargs)
        formdef_references = get_wcs_options('/api/formdefs/')
        self.fields['formdef_reference'].widget = forms.Select(choices=formdef_references)


class WcsCategoryCellForm(forms.ModelForm):
    class Meta:
        model = WcsCategoryCell
        fields = ('category_reference',)

    def __init__(self, *args, **kwargs):
        super(WcsCategoryCellForm, self).__init__(*args, **kwargs)
        references = get_wcs_options('/api/categories/')
        self.fields['category_reference'].widget = forms.Select(choices=references)



class MultiSortWidget(forms.SelectMultiple):
    def render(self, name, value, attrs=None, choices=()):
        # reorder choices to get them in the current value order
        self_choices = self.choices[:]
        choices_dict = dict(self_choices)
        if value:
            for option in reversed(value.get('data')):
                if not option in choices_dict:
                    continue
                option_tuple = (option, choices_dict[option])
                self.choices.remove(option_tuple)
                self.choices.insert(0, option_tuple)

        # render the <select multiple>
        rendered = super(MultiSortWidget, self).render(name, value,
                attrs=attrs, choices=choices)

        # include it in a <div> that will be turned into an appropriate widget
        # in javascript
        id_ = 'wid-%s' % name
        return mark_safe('''<div class="multisort" id="%s">%s</div>
        <script type="text/javascript">multisort($("#%s"));</script>
        ''' % (id_, rendered, id_))

    def render_options(self, choices, value):
        value = value.get('data') or []
        return super(MultiSortWidget, self).render_options(choices, value)

    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return {'data': data.getlist(name)}
        return data.get(name, None)


class WcsFormsOfCategoryCellForm(forms.ModelForm):
    class Meta:
        model = WcsFormsOfCategoryCell
        fields = ('category_reference', 'ordering', 'manual_order', 'limit')

    def __init__(self, *args, **kwargs):
        super(WcsFormsOfCategoryCellForm, self).__init__(*args, **kwargs)
        references = get_wcs_options('/api/categories/')
        formdef_references = get_wcs_options('/api/formdefs/', include_category_slug=True)
        self.fields['ordering'].widget = forms.Select(
                choices=self.fields['ordering'].choices,
                attrs={'class': 'ordering-select'})
        self.fields['category_reference'].widget = forms.Select(choices=references,
                attrs={'class': 'category-select'})
        self.fields['manual_order'].widget = MultiSortWidget(choices=formdef_references)

==> v1.51.zip: combo/combo/apps/wcs/forms.py <==
# combo - content management system
# Copyright (C) 2014-2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import django
from django import forms
from django.utils.datastructures import MultiValueDict
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from .models import (WcsFormCell, WcsCategoryCell, WcsFormsOfCategoryCell,
                     WcsCurrentFormsCell)
from .utils import get_wcs_options, get_wcs_services

class WcsFormCellForm(forms.ModelForm):
    class Meta:
        model = WcsFormCell
        fields = ('formdef_reference',)

    def __init__(self, *args, **kwargs):
        super(WcsFormCellForm, self).__init__(*args, **kwargs)
        formdef_references = get_wcs_options('/api/formdefs/')
        self.fields['formdef_reference'].widget = forms.Select(choices=formdef_references)


class WcsCategoryCellForm(forms.ModelForm):
    class Meta:
        model = WcsCategoryCell
        fields = ('category_reference',)

    def __init__(self, *args, **kwargs):
        super(WcsCategoryCellForm, self).__init__(*args, **kwargs)
        references = get_wcs_options('/api/categories/')
        self.fields['category_reference'].widget = forms.Select(choices=references)



class MultiSortWidget(forms.SelectMultiple):
    def render(self, name, value, attrs=None, choices=()):
        # reorder choices to get them in the current value order
        self_choices = self.choices[:]
        choices_dict = dict(self_choices)
        if value:
            for option in reversed(value.get('data')):
                if not option in choices_dict:
                    continue
                option_tuple = (option, choices_dict[option])
                self.choices.remove(option_tuple)
                self.choices.insert(0, option_tuple)

        # render the <select multiple>
        if django.VERSION < (1, 11, 0):
            rendered = super(MultiSortWidget, self).render(name, value,
                    attrs=attrs, choices=choices)
        else:
            rendered = super(MultiSortWidget, self).render(name, value,
                    attrs=attrs)

        # include it in a <div> that will be turned into an appropriate widget
        # in javascript
        id_ = 'wid-%s' % name
        return mark_safe('''<div class="multisort" id="%s">%s</div>
        <script type="text/javascript">multisort($("#%s"));</script>
        ''' % (id_, rendered, id_))

    def render_options(self, choices, value):
        value = value.get('data') or []
        return super(MultiSortWidget, self).render_options(choices, value)

    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return {'data': data.getlist(name)}
        return data.get(name, None)


class WcsFormsOfCategoryCellForm(forms.ModelForm):
    class Meta:
        model = WcsFormsOfCategoryCell
        fields = ('category_reference', 'ordering', 'manual_order', 'limit')

    def __init__(self, *args, **kwargs):
        super(WcsFormsOfCategoryCellForm, self).__init__(*args, **kwargs)
        references = get_wcs_options('/api/categories/')
        formdef_references = get_wcs_options('/api/formdefs/', include_category_slug=True)
        self.fields['ordering'].widget = forms.Select(
                choices=self.fields['ordering'].choices,
                attrs={'class': 'ordering-select'})
        self.fields['category_reference'].widget = forms.Select(choices=references,
                attrs={'class': 'category-select'})
        self.fields['manual_order'].widget = MultiSortWidget(choices=formdef_references)


class CategoriesSelectMultiple(forms.SelectMultiple):
    def format_value(self, value):
        # this converts data dictionary to list, for django >=1.11
        return super(CategoriesSelectMultiple, self).format_value(
                value.get('data') or [])

    def render_options(self, choices, value):
        # this converts data dictionary to list, for django <1.11
        value = value.get('data') or []
        return super(CategoriesSelectMultiple, self).render_options(choices, value)

    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return {'data': data.getlist(name)}
        return data.get(name, None)


class WcsCurrentFormsCellForm(forms.ModelForm):
    class Meta:
        model = WcsCurrentFormsCell
        fields = ['wcs_site', 'categories', 'current_forms', 'done_forms']

    def __init__(self, *args, **kwargs):
        super(WcsCurrentFormsCellForm, self).__init__(*args, **kwargs)
        if len(get_wcs_services()) == 1:
            self.fields['wcs_site'].widget = forms.HiddenInput()
        else:
            combo_wcs_sites = [('', _('All'))]
            wcs_sites = [(x, y.get('title')) for x, y in get_wcs_services().items()]
            wcs_sites.sort(key=lambda x: x[1])
            combo_wcs_sites.extend(wcs_sites)
            self.fields['wcs_site'].widget = forms.Select(choices=combo_wcs_sites,
                    attrs={'class': 'wcs-site-select'})
        categories = get_wcs_options('/api/categories/')
        self.fields['categories'].help_text = _('By default forms from all categories are displayed.')
        self.fields['categories'].widget = CategoriesSelectMultiple(choices=categories,
                attrs={'class': 'categories-select'})
